@@ -8,12 +8,13 @@ import re
 import logging
 import time
 import pkgutil
+import warnings
 from itertools import ifilter
 
 from requests import RequestException
 
 from flexget import config_schema
-from flexget.event import add_event_handler as add_phase_handler
+from flexget.event import add_event_handler as add_phase_handler, fire_event, remove_event_handlers
 from flexget import plugins as plugins_pkg
 
 log = logging.getLogger('plugin')
@@ -100,6 +101,7 @@ class PluginError(Exception):
         return self.value
 
 
+# TODO: move to utils or somewhere more appropriate
 class internet(object):
     """@internet decorator for plugin phase methods.
 
@@ -163,11 +165,7 @@ task_phases = ['start', 'input', 'metainfo', 'filter', 'download', 'modify', 'ou
 # map phase names to method names
 phase_methods = {
     # task
-    'abort': 'on_task_abort',  # special; not a task phase that gets called normally
-
-    # lifecycle
-    'process_start': 'on_process_start',
-    'process_end': 'on_process_end',
+    'abort': 'on_task_abort'  # special; not a task phase that gets called normally
 }
 phase_methods.update((_phase, 'on_task_' + _phase) for _phase in task_phases)  # DRY
 
@@ -177,22 +175,11 @@ plugins = {}
 # Loading done?
 plugins_loaded = False
 
-_parser = None
 _loaded_plugins = {}
 _plugin_options = []
 _new_phase_queue = {}
 
 
-def register_parser_option(*args, **kwargs):
-    """Adds a parser option to the global parser."""
-    if _parser is None:
-        import warnings
-        warnings.warn('register_parser_option called before it can be')
-        return
-    _parser.add_argument(*args, **kwargs)
-    _plugin_options.append((args, kwargs))
-
-
 def register_task_phase(name, before=None, after=None):
     """Adds a new task phase to the available phases."""
     if before and after:
@@ -214,11 +201,6 @@ def register_task_phase(name, before=None, after=None):
             task_phases.insert(task_phases.index(after) + 1, phase_name)
         if after is None:
             task_phases.insert(task_phases.index(before), phase_name)
-
-        # create possibly newly available phase handlers
-        for loaded_plugin in plugins:
-            plugins[loaded_plugin].build_phase_handlers()
-
         return True
 
     # if can't add yet (dependencies) queue addition
@@ -269,6 +251,10 @@ class PluginInfo(dict):
             # By default look at the containing package of the plugin.
             category = plugin_class.__module__.split('.')[-2]
 
+        # Check for unsupported api versions
+        if api_ver < 2:
+            warnings.warn('Api versions <2 are no longer supported. Plugin %s' % name, DeprecationWarning, stacklevel=2)
+
         # Set basic info attributes
         self.api_ver = api_ver
         self.name = name
@@ -279,12 +265,24 @@ class PluginInfo(dict):
         self.category = category
         self.phase_handlers = {}
 
-        # Create plugin instance
         self.plugin_class = plugin_class
+        self.instance = None
+
+        if self.name in plugins:
+            PluginInfo.dupe_counter += 1
+            log.critical('Error while registering plugin %s. %s' %
+                         (self.name, ('A plugin with the name %s is already registered' % self.name)))
+        else:
+            plugins[self.name] = self
+
+    def initialize(self):
+        if self.instance is not None:
+            # We already initialized
+            return
+        # Create plugin instance
         self.instance = self.plugin_class()
         self.instance.plugin_info = self  # give plugin easy access to its own info
         self.instance.log = logging.getLogger(getattr(self.instance, "LOGGER_NAME", None) or self.name)
-
         if hasattr(self.instance, 'schema'):
             self.schema = self.instance.schema
         elif hasattr(self.instance, 'validator'):
@@ -298,13 +296,7 @@ class PluginInfo(dict):
             self.schema['id'] = location
             config_schema.register_schema(location, self.schema)
 
-        if self.name in plugins:
-            PluginInfo.dupe_counter += 1
-            log.critical('Error while registering plugin %s. %s' %
-                         (self.name, ('A plugin with the name %s is already registered' % self.name)))
-        else:
-            self.build_phase_handlers()
-            plugins[self.name] = self
+        self.build_phase_handlers()
 
     def reset_phase_handlers(self):
         """Temporary utility method"""
@@ -347,17 +339,16 @@ class PluginInfo(dict):
     __repr__ = __str__
 
 
-register_plugin = PluginInfo
+register = PluginInfo
 
 
 def _strip_trailing_sep(path):
     return path.rstrip("\\/")
 
 
-def get_standard_plugins_path():
+def _get_standard_plugins_path():
     """
     :returns: List of directories where plugins should be tried to load from.
-    :rtype: list
     """
     # Get basic path from environment
     env_path = os.environ.get('FLEXGET_PLUGIN_PATH')
@@ -420,25 +411,23 @@ def _load_plugins_from_dirs(dirs):
                       'point (before, after). Plugin is not working properly.' % (args[0], phase))
 
 
-def load_plugins(parser):
+def load_plugins():
     """Load plugins from the standard plugin paths."""
-    global plugins_loaded, _parser
-
-    if plugins_loaded:
-        if parser is not None:
-            for args, kwargs in _plugin_options:
-                parser.add_argument(*args, **kwargs)
+    global plugins_loaded
 
     # suppress DeprecationWarning's
-    import warnings
     warnings.simplefilter('ignore', DeprecationWarning)
 
     start_time = time.time()
-    _parser = parser
-    try:
-        _load_plugins_from_dirs(get_standard_plugins_path())
-    finally:
-        _parser = None
+    # Import all the plugins
+    _load_plugins_from_dirs(_get_standard_plugins_path())
+    # Register them
+    fire_event('plugin.register')
+    # Plugins should only be registered once, remove their handlers after
+    remove_event_handlers('plugin.register')
+    # After they have all been registered, instantiate them
+    for plugin in plugins.values():
+        plugin.initialize()
     took = time.time() - start_time
     plugins_loaded = True
     log.debug('Plugins took %.2f seconds to load' % took)
@@ -477,7 +466,9 @@ def plugin_schemas(**kwargs):
     """Create a dict schema that matches plugins specified by `kwargs`"""
     return {'type': 'object',
             'properties': dict((p.name, {'$ref': p.schema['id']}) for p in get_plugins(**kwargs)),
-            'additionalProperties': False}
+            'additionalProperties': False,
+            'error_additionalProperties': '{{message}} Only known plugin names are valid keys.',
+            'patternProperties': {'^_': {'title': 'Disabled Plugin'}}}
 
 
 config_schema.register_schema('/schema/plugins', plugin_schemas)
@@ -490,6 +481,7 @@ def get_plugins_by_phase(phase):
 
     Return an iterator over all plugins that hook :phase:
     """
+    warnings.warn('Deprecated API', DeprecationWarning, stacklevel=2)
     if not phase in phase_methods:
         raise Exception('Unknown phase %s' % phase)
     return get_plugins(phase=phase)
@@ -507,6 +499,7 @@ def get_plugins_by_group(group):
 
     Return an iterator over all plugins with in specified group.
     """
+    warnings.warn('Deprecated API', DeprecationWarning, stacklevel=2)
     return get_plugins(group=group)
 
 
