@@ -1,6 +1,8 @@
 from __future__ import unicode_literals, division, absolute_import
 import logging
-from flexget.plugin import priority, register_plugin
+
+from flexget import plugin
+from flexget.event import event
 
 log = logging.getLogger('torrent_size')
 
@@ -10,8 +12,8 @@ class TorrentSize(object):
     Provides file size information when dealing with torrents
     """
 
-    @priority(200)
-    def on_task_modify(self, task):
+    @plugin.priority(200)
+    def on_task_modify(self, task, config):
         for entry in task.entries:
             if 'torrent' in entry:
                 size = entry['torrent'].size / 1024 / 1024
@@ -19,4 +21,6 @@ class TorrentSize(object):
                 entry['content_size'] = size
 
 
-register_plugin(TorrentSize, 'torrent_size', builtin=True)
+@event('plugin.register')
+def register_plugin():
+    plugin.register(TorrentSize, 'torrent_size', builtin=True, api_ver=2)
